@@ -1,13 +1,31 @@
-# $Header: /raid/cvsroot/DBIx/DBIx-SearchBuilder/SearchBuilder/Handle/mysql.pm,v 1.5 2001/01/25 03:06:31 jesse Exp $
+# $Header: /raid/cvsroot/DBIx/DBIx-SearchBuilder/SearchBuilder/Handle/mysql.pm,v 1.7 2001/05/23 05:10:14 jesse Exp $
 
 package DBIx::SearchBuilder::Handle::mysql;
 use DBIx::SearchBuilder::Handle;
 @ISA = qw(DBIx::SearchBuilder::Handle);
 
 use vars qw($VERSION @ISA $DBIHandle $DEBUG);
-
-
 use strict;
+
+=head1 NAME
+
+  DBIx::SearchBuilder::Handle::mysql -- a mysql specific Handle object
+
+=head1 SYNOPSIS
+
+
+=head1 DESCRIPTION
+
+=head1 AUTHOR
+
+Jesse Vincent, jesse@fsck.com
+
+=head1 SEE ALSO
+
+perl(1), DBIx::SearchBuilder
+
+=cut
+
 # {{{ sub Insert
 
 =head2 Insert
@@ -31,6 +49,11 @@ sub Insert  {
      }
 
     $self->{'id'}=$self->dbh->{'mysql_insertid'};
+    # Yay. we get to work around mysql_insertid being null some of the time :/
+ 
+    unless ($self->{'id'}) {
+	$self->{'id'} =  $self->FetchResult('SELECT LAST_INSERT_ID()');
+    }
     warn "$self no row id returned on row creation" unless ($self->{'id'});
     return( $self->{'id'}); #Add Succeded. return the id
   }
@@ -39,21 +62,3 @@ sub Insert  {
 
 
 
-=head1 NAME
-
-  DBIx::SearchBuilder::Handle::mysql -- a mysql specific Handle object
-
-=head1 SYNOPSIS
-
-
-  =head1 DESCRIPTION
-
-=head1 AUTHOR
-
-Jesse Vincent, jesse@fsck.com
-
-=head1 SEE ALSO
-
-perl(1), DBIx::SearchBuilder
-
-=cut
